#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>

#define MAX_LIN    3000   //--- Mximo tamao de una lnea
#define MAX_IP      100   //--- Mximo tamao de una direccin ip
#define MAX_FICH    400   //--- Mximo tamao del nombre de un fichero
#define MAX_CAMPOS 2600   //--- Mximo tamao del resto de campos

#define SEG_TIME   1041375600  // --- 1 de enero de 2003 a las 00:00:00 para ctime()

//--- Definicin de Parmetros variables ---
long n_lin=0;       // --- Nmero de lineas
long n_ip=0;        // --- Nmero de direcciones ip distintas
long n_fich=0;      // --- Nmero de ficheros distintos
long n_dias=0;      // --- Nmero de das

long d_lin=0;	// --- Nmero de lneas ledas
long d_ip=0;	// --- Nmero de ips distintas leidas
long d_fich=0;   // --- Nmero de ficheros distintos leidos

char **direc;        // --- Array de Direcciones
char **ficheros;     // --- Array de Nombres de Ficheros
char **campos;       // --- Array de Resto de campos


//----------------------- mostrar_sintaxis --------------------------

void mostrar_sintaxis(){
	printf("Sintaxis: generador param1 param2 param3 param4 param5 param6\n");
	printf("Parmetro 1: Nombre del fichero log de entrada\n");
	printf("Parmetro 2: Nombre del fichero log de salida a generar\n");
	printf("Parmetro 3: Nmero de lneas del fichero de salida\n");
	printf("Parmetro 4: Nmero de direcciones ip distintas\n");
	printf("Parmetro 5: Nmero de nombres de ficheros distintos\n");
	printf("Parmetro 6: Nmero de das distintos)\n");
}

/*---------------------- cortar_campo ---------------------*/
// --- Esta funcin corta de una cadena una subcadena hasta el siguiente
// --- caracter delimitador
/*---------------------------------------------------------*/

int cortar_campo (char *cadena, char delimitador, long MAX, char *campo){

     int i;

     /* --- leer el campo --- */
     for (i=0; (i<strlen(cadena)) && (cadena[i] != delimitador) && (cadena[i] != '\n') && (i<MAX); i++)
            campo[i] = cadena[i];

     campo[i] ='\0';

     if (i!=0) return(i+1);
        else return(-1);
}

// ---------------------- insertar_sin_repeticiones -------------------------------
// Esta funcin inserta una cadena en un array de cadenas en la ltima posicin.
// Se comprueba que no existan repeticiones.
// --------------------------------------------------------------------------------

int insertar_sin_repeticiones(char **array, char *elemento, int pos){

	bool encontrado=false;
	int i=0;

	while ((i<pos) && !encontrado){
		if (strcmp(array[i],elemento) == 0)
			encontrado=true;
		i++;
	}

	if (!encontrado){
		array[pos]=elemento;
		return(1);
	}
	else return(0);
}


// ---------------------- convertir fecha ----------------------
// --- Esta funcin convierte segundos a DD/MES/AAAA:HH:MM:SS
// -------------------------------------------------------------

 void convertir_fecha(char *f, char *fecha){

	char dia[3], mes[4], anno[5], hora[9];

	strncpy(dia,&f[8],2);
	dia[2]='\0';
	if (atoi(dia)<10) dia[0]='0';
	strncpy(mes,&f[4],3);
	mes[3]='\0';
	strncpy(anno,&f[20],4);
	anno[4]='\0';
	strncpy(hora,&f[11],8);
	hora[8]='\0';

	sprintf(fecha, "%s/%s/%s:%s",dia, mes, anno, hora);

}


// ----------------------- comprobar_param------------------------------

bool comprobar_param(int argc, char **argv){

   	bool error=false;

//--- Comprobacin del nmero de parmetros
   if (argc != 7){
	printf("\nError: Nmero de parmetros incorrecto\n\n");
	mostrar_sintaxis();
	error=true;
    }
    else {

    //--- Comprobacin existe fichero de entrada y permisos
    if (access(argv[1], R_OK) == -1){
	printf("\nError: No existe el fichero log de entrada o no tiene permisos\n\n");
	mostrar_sintaxis();
	error=true;
    }

    //--- Comprobacin n de lineas
    if (((n_lin=atol(argv[3])) == -1) || (n_lin<0)){
	printf("\nError: El parmetro 3 debe ser un nmero entero positivo\n\n");
	mostrar_sintaxis();
	error=true;
    }

    //--- Comprobacin n de direcciones ip distintas
    if (((n_ip=atol(argv[4])) == -1) || (n_ip > n_lin) || (n_ip<0)){
	printf("\nError: El parmetro 4 debe ser un nmero entero positivo\n");
	printf("Error: El parmetro 4 debe ser menor o igual que el parmetro 3\n\n");
	mostrar_sintaxis();
	error=true;
    }

    //--- Comprobacin n de ficheros distintos
    if (((n_fich=atol(argv[5])) == -1) || (n_fich > n_lin) || (n_fich<0)){
	printf("\nError: El parmetro 5 debe ser un nmero entero positivo\n");
	printf("Error: El parmetro 5 debe ser menor o igual que el parmetro 3\n\n");
	mostrar_sintaxis();
	error=true;
    }

    //--- Comprobacin n de das distintos
    if (((n_dias=atol(argv[6])) == -1) || (n_dias <0) ){
	printf("\nError: El parmetro 6 debe ser un nmero positivo\n");
	mostrar_sintaxis();
	error=true;
    }
   }

   if (!error){
       // --- Muestra los parmetros por pantalla
    printf("\nFichero log de entrada: %s\n",argv[1]);
    printf("Fichero log de salida: %s\n",argv[2]);
    printf("N Lneas: %d\n", n_lin);
    printf("N Direcciones ip: %d\n", n_ip);
    printf("N Ficheros: %d\n", n_fich);
    printf("N Dias: %d\n", n_dias);

    direc = new char * [n_ip];          // --- Direcciones
    ficheros = new char * [n_fich];     // --- Nombres de Ficheros
    campos = new char * [n_lin];        // --- Resto de campos
    }
    return(!error);
}

// --------------------------- leer_fich_entrada ----------------------------

void leer_fich_entrada(char *nom_fich_ent){

    FILE *fich_log;  //--- Fichero de entrada

    // ---- Definicin de las variables para cortar los campos
    char linea[MAX_LIN]; //--- Lnea del fichero de entrada

    char c_ip[MAX_IP];          //--- Campo 1: Direccin IP
    char c_fichero[MAX_FICH];   //--- Campo 3: Fichero
    char c_resto_campos[MAX_CAMPOS];   //--- Resto de campos

    char * dir_ip;   //--- Direccin IP
    char * fichero;  //--- Fichero
    char * resto_campos; //--- Resto de campos

    int pos=0;    //--- Posicin donde se encuentra el campo siguiente
    int avance=0; //--- Longitud del campo leido

    long conta_lin=0;   // --- Nmero de lneas leidas

    // --- Se abre el fichero de entrada
    if ((fich_log = fopen(nom_fich_ent,"r")) != NULL){

        // --- Se recorre el fichero de entrada
        while ((fgets(linea, MAX_LIN, fich_log)) && ((d_ip<n_ip)||(d_fich<n_fich)||(d_lin<n_lin))){
		   pos=0;
		   conta_lin++;

		   // --- Leer campo 1: direccin IP (delimitador espacio en blanco)
		   avance=cortar_campo(linea,' ',MAX_IP,c_ip);
		   pos=avance + 34; // 34 caracteres de hasta el campo del fichero
		   if (d_ip < n_ip){
		   	dir_ip = new char[strlen(c_ip)+1];
		   	strcpy(dir_ip,c_ip);
		   	if(insertar_sin_repeticiones(direc, dir_ip, d_ip))
		   		d_ip++;
		   }

		   // --- Leer campo 3: fichero (delimitador comillas " (ascii 34))
		   avance=cortar_campo(&linea[pos],34,MAX_FICH,c_fichero);
		   pos=pos+avance+1;
		   if (d_fich < n_fich){
		   	fichero = new char[strlen(c_fichero)+1];
		   	strcpy(fichero,c_fichero);
		   	if(insertar_sin_repeticiones(ficheros, fichero, d_fich))
		   		d_fich++;
		   }

		   // --- Leer resto de campos
		   avance=cortar_campo(&linea[pos],'\n',MAX_CAMPOS,c_resto_campos);
		   pos=pos+avance;
		   if (d_lin < n_lin){
		      resto_campos = new char[strlen(c_resto_campos)+1];
		      strcpy(resto_campos, c_resto_campos);
  		      campos[d_lin]=resto_campos;
		      d_lin++;
		    }
	}

	printf("------------------------------\n");
	printf("Nmero de Lneas ledas del fichero de entrada: %d\n", conta_lin);
	printf("Nmero de Lneas almacendas: %d\n", d_lin);
	printf("Nmero de Direcciones Ip distintas: %d\n", d_ip);
	printf("Nmero de Ficheros ledos distintos: %d\n", d_fich);
        fclose(fich_log);
  }
}

// ------------------------- escribir_fich_salida -----------------------

void escribir_fich_salida(char *nom_fich_sal){

        FILE *fich_sal;  //--- Fichero de salida

	long pos_ip=-1;      //Indice del array de direcciones IP
	bool pri_ip=true;    // Indicador de la primera escritura
	long pos_fich=-1;    // Indice del array de nombres de ficheros
	bool pri_fich=true;  // Indicador de la primera escritura
	long pos_campos=-1;    // Indice del array de nombres de ficheros
	bool pri_campos=true;  // Indicador de la primera escritura

	char fecha[30];  // Fecha de la peticin
	time_t n_seg=SEG_TIME;  // Nmero de segundos de cada peticin
	int paso_seg=0;  // Nmero de segundos que se incrementan en cada peticin
	long t_seg=0;    // Nmero de segundos totales de los n_dias

   	int i; // --- Variable auxiliar

	t_seg= n_dias * (60*60*24) -1;
	paso_seg= (int) ((double) t_seg / (double) d_lin);

	//--- Creacin del fichero de salida
        if ((fich_sal = fopen(nom_fich_sal,"w")) != NULL){

	for (i=0; i<n_lin; i++){

	// --- Clculo de la direccin ip
	if (!pri_ip)
		// Generacin aleatoria del ndice
		pos_ip= (long) ( ((double) rand() /RAND_MAX) * d_ip);
	else    // Escritura de los primeros n_ip direcciones
		if (pos_ip<(d_ip-1)) pos_ip++;
		else pri_ip=false;

	// --- Clculo del nombre fichero
	if (!pri_fich)
	        //Generacin aleatoria del ndice
		pos_fich= (long) ( ((double) rand() /RAND_MAX) * d_fich);
	else    // Escritura de los primeros n_fich nombres de fichero
		if (pos_fich<(d_fich-1))  pos_fich++;
		else pri_fich=false;

	// --- Clculo de la FECHA
	n_seg = n_seg + paso_seg;  //--- incremento de segundos
	convertir_fecha(ctime(&n_seg), fecha);

	// --- Clculo del resto de campos
	if (!pri_campos)
	        //Generacin aleatoria del ndice
		pos_campos= (long) ( ((double) rand() /RAND_MAX) * d_lin);
	else    // Escritura de los primeros n_fich nombres de fichero
		if (pos_campos<(d_lin-1))  pos_campos++;
		else pri_campos=false;


	// --- Escribir en el fichero de salida
	fprintf(fich_sal, "%s - - [%s +0200] \"%s\" %s\n", direc[pos_ip], fecha, ficheros[pos_fich], campos[pos_campos]);

	}
	fclose(fich_sal);
	printf("\nFichero creado %s.  N Lneas: %d\n\n", nom_fich_sal, i);
	}

	//--- Liberar memoria
        for (i=0; i<n_ip; i++) delete [] direc[i];
	for (i=0; i<n_fich; i++) delete [] ficheros[i];
	for (i=0; i<n_lin; i++) delete [] campos[i];
	delete [] direc;
        delete [] ficheros;
        delete [] campos;
}

/* ---------------------- main ------------------------- */

main(int argc, char *argv[])
{
    // --- FASE 0: Comprobacin de errores en los parmetros
    if (comprobar_param(argc, argv)){  //--- Los parmetros son correctos

    	// --- FASE 1: Recorrer el fichero de entrada y cargar datos en memoria
    	leer_fich_entrada(argv[1]);

    	// --- FASE 2: Generar fichero de salida
    	escribir_fich_salida(argv[2]);

    }
}
